/* xcomplex.h -- complex class for extended precision bcd numbers.
 * (C) 2001, C. Bond. All rights reserved.
 */
#ifndef xcomplexH
#define xcomplexH

#include <iostream.h>
#include "xfloat.h"


class xcomplex  {
    bcd32 xreal,ximag;
public:
// Constructors
    xcomplex (bcd32=k0,bcd32=k0);
    xcomplex (char *x,char *y);
    xcomplex (double x,double y=0.0);
    xcomplex (xfloat x,xfloat y);
// Unary operators
    xcomplex operator+();
    xcomplex operator-();
// Binary operators
// +
    xcomplex operator+(xcomplex op2);
    xcomplex operator+(xfloat z);
    xcomplex operator+(double z);
    friend xcomplex operator+(xfloat z,xcomplex op2);
    friend xcomplex operator+(double z,xcomplex op2);
// -
    xcomplex operator-(xcomplex op2);
    xcomplex operator-(xfloat z);
    xcomplex operator-(double z);
    friend xcomplex operator-(xfloat z,xcomplex op2);
    friend xcomplex operator-(double z,xcomplex op2);
// *
    xcomplex operator*(xcomplex op2);
    xcomplex operator*(xfloat z);
    xcomplex operator*(double z);
    friend xcomplex operator*(xfloat z,xcomplex op2);
    friend xcomplex operator*(double z,xcomplex op2);
// /
    xcomplex operator/(xcomplex op2);
    xcomplex operator/(xfloat z);
    xcomplex operator/(double z);
    friend xcomplex operator/(xfloat z,xcomplex op2);
    friend xcomplex operator/(double z,xcomplex op2);
// =
    xcomplex operator=(xcomplex op2);
    xcomplex operator=(xfloat z);
    xcomplex operator=(double z);
// +=
    xcomplex operator+=(xcomplex op2);
    xcomplex operator+=(xfloat z);
    xcomplex operator+=(double z);
// -=
    xcomplex operator-=(xcomplex op2);
    xcomplex operator-=(xfloat z);
    xcomplex operator-=(double z);
// *=
    xcomplex operator*=(xcomplex op2);
    xcomplex operator*=(xfloat z);
    xcomplex operator*=(double z);
// /=
    xcomplex operator/=(xcomplex op2);
    xcomplex operator/=(xfloat z);
    xcomplex operator/=(double z);
// Relational operators
    bool operator==(xcomplex op2);
    bool operator!=(xcomplex op2);
// Setters and getters
    friend xfloat real(xcomplex a);
    friend xfloat imag(xcomplex a);
    void real(xfloat);
    void imag(xfloat);
// Math functions
    friend xfloat mag(xcomplex a);
    friend xfloat abs(xcomplex a);
    friend double dabs(xcomplex a);
    friend xfloat norm(xcomplex a);
    friend xcomplex sqrt(xcomplex a);
    friend xcomplex polar(xcomplex a);
    friend xcomplex rect(xcomplex a);
    friend xcomplex exp(xcomplex a);
    friend xcomplex log(xcomplex a);
    friend xcomplex pow(xcomplex a,xcomplex b);
    friend xcomplex sin(xcomplex a);
    friend xcomplex cos(xcomplex a);
    friend xcomplex tan(xcomplex a);
    friend xcomplex csc(xcomplex a);
    friend xcomplex sec(xcomplex a);
    friend xcomplex cot(xcomplex a);
    friend xcomplex asin(xcomplex a);
    friend xcomplex acos(xcomplex a);
    friend xcomplex atan(xcomplex a);
    friend xcomplex acsc(xcomplex a);
    friend xcomplex asec(xcomplex a);
    friend xcomplex acot(xcomplex a);
    friend xcomplex sinh(xcomplex a);
    friend xcomplex cosh(xcomplex a);
    friend xcomplex tanh(xcomplex a);
    friend xcomplex csch(xcomplex a);
    friend xcomplex sech(xcomplex a);
    friend xcomplex coth(xcomplex a);
// I/O operators
    friend ostream &operator<<(ostream &stream,xcomplex a);
    friend istream &operator>>(istream &stream,xcomplex &a);
// Misc functions
};
#endif
